@@ -43,20 +43,6 @@ impl<FE> FileState<FE> {
             _ => false,
         }
     }
-
-    fn size(&self) -> Option<usize> {
-        match self {
-            Self::Pending => None,
-            Self::Read(size, _) | Self::Modified(size, _) => Some(*size),
-        }
-    }
-
-    fn maybe_lock_contents(&self) -> Option<OwnedRwLockWriteGuard<FE>> {
-        match self {
-            Self::Pending => None,
-            Self::Read(_, lock) | Self::Modified(_, lock) => lock.clone().try_write_owned().ok(),
-        }
-    }
 }
 
 struct Inner<FE> {
@@ -108,8 +94,13 @@ impl<FE> FileLock<FE> {
         }
     }
 
-    pub(crate) fn size(&self) -> Option<usize> {
-        let state = self.inner.contents.try_read().ok()?;
+    pub fn path(&self) -> &Path {
+        self.inner.path.as_path()
+    }
+
+    pub async fn size_hint(&self) -> Option<usize> {
+        let state = self.inner.contents.read().await;
+
         match &*state {
             FileState::Pending => None,
             FileState::Read(size, _) => Some(*size),
@@ -117,7 +108,7 @@ impl<FE> FileLock<FE> {
         }
     }
 
-    async fn get_lock(&self) -> Result<Arc<RwLock<FE>>, io::Error>
+    async fn get_lock(&self, mutate: bool) -> Result<Arc<RwLock<FE>>, io::Error>
     where
         FE: FileLoad,
     {
@@ -143,12 +134,20 @@ impl<FE> FileLock<FE> {
             *file_state = FileState::Read(size, Arc::new(RwLock::new(entry)));
         }
 
-        let file_state = file_state.downgrade();
+        if mutate {
+            let new_state = match &*file_state {
+                FileState::Pending => unreachable!(),
+                FileState::Read(size, lock) => FileState::Modified(*size, lock.clone()),
+                FileState::Modified(size, lock) => FileState::Modified(*size, lock.clone()),
+            };
+
+            *file_state = new_state;
+        }
+
         match &*file_state {
             FileState::Pending => unreachable!(),
             FileState::Read(size, contents) | FileState::Modified(size, contents) => {
                 self.inner
-                    .clone()
                     .cache
                     .insert(self.inner.path.clone(), self.clone(), *size);
 
@@ -162,7 +161,7 @@ impl<FE> FileLock<FE> {
     where
         FE: FileLoad + AsType<F>,
     {
-        let contents = self.get_lock().await?;
+        let contents = self.get_lock(false).await?;
         let guard = contents.read_owned().await;
         OwnedRwLockReadGuard::try_map(guard, |entry| entry.as_type())
             .map(|guard| FileReadGuard { guard })
@@ -179,8 +178,9 @@ impl<FE> FileLock<FE> {
     where
         FE: FileLoad + AsType<F>,
     {
-        let contents = self.get_lock().await?;
+        let contents = self.get_lock(true).await?;
         let guard = contents.write_owned().await;
+
         OwnedRwLockWriteGuard::try_map(guard, |entry| entry.as_type_mut())
             .map(|guard| FileWriteGuard { guard })
             .map_err(|_| {
@@ -223,21 +223,42 @@ impl<FE> FileLock<FE> {
         Ok(())
     }
 
-    pub(crate) fn evict(self) -> Option<impl Future<Output = Result<(), io::Error>>>
+    pub(crate) fn evict(self) -> Option<(usize, impl Future<Output = Result<(), io::Error>>)>
     where
         FE: FileLoad + 'static,
     {
         let mut state = self.inner.contents.clone().try_write_owned().ok()?;
 
-        let old_size = state.size()?;
-        let contents = state.maybe_lock_contents()?;
+        let (old_size, contents, modified) = match &*state {
+            FileState::Pending => return None,
+            FileState::Read(size, contents) => {
+                let contents = contents.clone().try_write_owned().ok()?;
+                (*size, contents, false)
+            }
+            FileState::Modified(size, contents) => {
+                let contents = contents.clone().try_write_owned().ok()?;
+                (*size, contents, true)
+            }
+        };
+
+        let eviction = async move {
+            if modified {
+                persist(self.inner.path.as_path(), &*contents).await?;
+            }
+
+            self.inner.cache.lfu.remove(&self.inner.path);
+            self.inner
+                .cache
+                .inner
+                .lock()
+                .expect("file cache state")
+                .size -= old_size;
 
-        Some(async move {
-            persist(self.inner.path.as_path(), &*contents).await?;
-            self.inner.cache.remove(&self.inner.path, old_size);
             *state = FileState::Pending;
             Ok(())
-        })
+        };
+
+        Some((old_size, eviction))
     }
 }
 
